from __future__ import annotations
from dataclasses import dataclass, field
from datetime import datetime, date
from typing import Any, Dict, List, Tuple


@dataclass(repr=True, eq=True, frozen=True)
class Attachment:
    """添付ファイルデータ"""

    name: str
    """ファイル名"""
    url: str
    """URL"""

    def __repr__(self) -> str:
        name = self.name
        if len(name) > 20:
            name = name[0:17] + "..."
        return f"Attachment({name})"


@dataclass(repr=True, eq=True, frozen=True)
class Post:
    """
    掲示板の投稿
    """

    post_id: int
    """投稿ID（内部処理用）"""
    title: str
    """タイトル"""
    description: str
    """本文"""
    update_date: date
    """最終更新日"""
    show_date: Tuple[date, date]
    """公開期間"""
    author: str
    """担当者"""
    origin: str
    """発信元"""
    category: str
    """カテゴリー"""
    attachment: List[Attachment]
    """添付ファイル"""
    other: Dict[str, str]
    """その他のフィールド"""

    def __repr__(self) -> str:
        title = self.title
        title = title.replace("\n", " ")
        if len(title) > 20:
            title = title[0:17] + "..."
        if len(self.attachment) > 0:
            return f"Post({title}, 添付ファイルあり)"
        return f"Post({title})"

    @staticmethod
    def parse_post(post_raw: Dict) -> Post:
        """取得された生データから投稿情報のクラスに変換します

        Parameters
        ----------
        post_raw : dict
            取得されたJSONのデータ

        Returns
        -------
        Post
            投稿情報
        """

        post_data_raw: dict[str, str] = post_raw["data"]

        # 投稿ID
        post_id = post_raw["id"]
        # タイトル
        title = post_data_raw.pop("タイトル")
        # 本文
        description = post_data_raw.pop("本文")
        # 最終更新日
        update_date_raw = post_data_raw.pop("最終更新日")
        update_date = datetime.strptime(update_date_raw[:-5], "%Y/%m/%d").date()
        # 公開期間
        show_date_raw = post_data_raw.pop("公開期間")
        show_date_start_raw, show_date_end_raw = show_date_raw.split(" 〜 ")
        show_date_start = datetime.strptime(show_date_start_raw[:-5], "%Y/%m/%d").date()
        show_date_end = datetime.strptime(show_date_end_raw[:-5], "%Y/%m/%d").date()
        show_date = (show_date_start, show_date_end)
        # 担当者
        author = post_data_raw.pop("担当者")
        # 発信元
        origin = post_data_raw.pop("発信元")
        # カテゴリー
        category = post_data_raw.pop("カテゴリー")
        # 添付ファイル
        attachment_raw = (
            post_data_raw.pop("添付ファイル") if "添付ファイル" in post_data_raw else None
        )
        attachment = []
        if attachment_raw is not None:
            for s in attachment_raw.split("\n"):
                name, url = s[1:-1].split("](")
                attachment.append(Attachment(name=name, url=url))

        return Post(
            post_id=post_id,
            title=title,
            description=description,
            update_date=update_date,
            show_date=show_date,
            author=author,
            origin=origin,
            category=category,
            attachment=attachment,
            other=post_data_raw,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tuat_feed']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'tuat-feed',
    'version': '0.2.0',
    'description': 'Unofficial library for fetching the feed for TUAT',
    'long_description': '# (非公式)TUAT掲示板ライブラリ\n\n## インストール\n* `python` >= 3.7\n\n```sh\n$ pip install tuat-feed\n```\n\n## 使い方\n\n```python\n>>> import tuat_feed\n>>> feed = tuat_feed.fetch()\n>>> len(feed)\n40\n>>> feed[0]\nPost(...)\n```\n\n### fetch\n`fetch()`を使って投稿情報をダウンロードします。結果は`Post`の配列になっています。\n### Post\n\n`Post`の定義はかんたんに書くと次のようになっています。\n\n```python\nclass Post:\n    post_id: int  # 投稿ID（内部処理用）\n    title: str  # タイトル\n    description: str  # 本文\n    update_date: date  # 最終更新日\n    show_date_start: date  # 公開期間（開始）\n    show_date_end: date  # 公開期間（終了）\n    author: str  # 担当者\n    origin: str  # 発信元\n    category: str  # カテゴリー\n    attachment: List[Attachment]  # 添付ファイル\n    other: Dict[str, str]  # その他のフィールド\n```\n\n### Attachment\n\n`Attachment`の定義はかんたんに書くと次のようになっています。\n\n```python\nclass Attachment:\n    name: str  # ファイル名\n    url: str  # URL\n```',
    'author': 'Shogo Takata',
    'author_email': 's196643z@st.go.tuat.ac.jp',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pineapplehunter/tuat-feed',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import List
from tuat_feed.post import Post
import requests


def fetch(
    category: str = "all", url: str = "https://api.ihavenojob.work/tuat/"
) -> List[Post]:
    """投稿の取得

    Parameters
    ----------
    category : str, optional
        取得する投稿の種類。"all", "campus", "academic"。デフォルトでは"all"。
    url : str, optional
        データを取得するAPIのURL、デフォルトでは"https://api.ihavenojob.work/tuat/"

    Returns
    -------
    List[Post]
        投稿の一覧

    Raises
    ------
    ValueError
        categoryに規定外の入力があった場合
    """

    if category == "all":
        response = requests.get(url)
    elif category == "academic":
        response = requests.get(url + "academic")
    elif category == "campus":
        response = requests.get(url + "campus")
    else:
        raise ValueError(
            f'category must be one of "all", "academic" or "campus". {category} was supplied.'
        )

    feed = response.json()
    posts = []
    for p in feed:
        posts.append(Post.parse_post(p))
    return posts
